import bpy
import bmesh
from mathutils import Vector


class Relax_Operator(bpy.types.Operator):
    bl_idname = "view3d.relax_operator"
    bl_label = "Simple operator"
    bl_description = "Center 3d cursor"


    def execute(self, context):


        # You can change these variables if you are unsatisfied with the results
        REPEAT = 100
        STRENGTH = 0.1


        ob = bpy.context.active_object

        if ob.mode == "EDIT":
            bm = bmesh.from_edit_mesh(ob.data)
        else:
            bm = bmesh.new()
            bm.from_mesh(ob.data)

        selected_verts = [vert for vert in bm.verts if vert.select]

        for _ in range(REPEAT):
            new_co = []
            for vert in selected_verts:
                co = Vector()
                for other in (e.other_vert(vert) for e in vert.link_edges):
                    co += other.co
                co /= len(vert.link_edges)
                co -= vert.co
                co -= co.dot(vert.normal) * vert.normal
                co += vert.co
                new_co.append(co)

            bm.normal_update()

            for vert, co in zip(selected_verts, new_co):
                vert.co = vert.co * (1 - STRENGTH) + co * STRENGTH

        if ob.mode == "EDIT":
            bmesh.update_edit_mesh(ob.data)
        else:
            bm.to_mesh(ob.data)


        return{'FINISHED'}
